function edgep =  edges_matlab(p,H,K)

% This function computes the edges of a polytop
% from its the vertices p (of dimension n*dim 
% where dim = 3)
%
% CALL :  edgep =  edges_matlab(p,H,K)
%         (H,K) is the H-representation of the polytop,
%         p the extrem points of the polytop
%         if H or K is empty, this representation is computed


if(isempty(H) | isempty(K))
 [H,K]    = polytope_V2H(p);
 p        = polytope_H2V(H,K);
end


np     = size(p,1); 
dim    = size(p,2);
nmax   = 1e6;

if (dim~=3 | np>nmax )
    error('Problem of dimension in edges.m');
end

Kt     = convhulln(p);
liste  = [Kt(:,1),Kt(:,2);Kt(:,1),Kt(:,3);Kt(:,2),Kt(:,3)];
liste  = sort(liste,2);
liste  = unique(liste,'rows');

pm     = (p(liste(:,1),:)+p(liste(:,2),:))/2;

nlm    = size(pm,1);
edgep  = zeros(nlm,2);
for  k = 1:nlm
  smv  = H*(pm(k,:)')-K;
  indice = find(abs(smv)<=1e-10);
  if(length(indice)==2)
    edgep(k,:) = liste(k,:);
  end
end

edgep  = edgep(find(edgep(:,1)~=0),:);
