function [xc,R] = inscribe_ball(p,H,K)


% This function computes center and radius of 
% the largest ball inscribed in a convex  polytop defined
% by the vertices p.
% 
% CALL : [center,Radius] = inscribe_ball(p,H,K)
%
% Remark : Uncomment the lines 32-34 and 43 if you want to use the
% cddmex linear solver

if(isempty(H) | isempty(K))
 [H,K]    = polytope_V2H(p);
 p        = polytope_H2V(H,K);
end
np        = size(p,1);  
dim       = size(p,2);



if (dim<2 | dim>3)
    error('Problem of dimension in inscribe_ball.m');
end


Aconstr                  = [H, -sqrt(sum(H.*H,2))];

[xopt,fval,exitflag]     = linprog([zeros(dim,1);1],Aconstr,K);
IN.A                     = Aconstr;
IN.B                     = K;
IN.obj                   = [zeros(dim,1);1]';
IN.lin                   = []; 
%OUT                      = cddmex('solve_lp',IN);
%xopt                     = OUT.xopt;
%exitflag                 = OUT.how;

if(exitflag==1)
  % Center of the ball
  xc                       = xopt(1:dim); 

  % Radius of the ball
%  R                        = OUT.objlp;
   R                       = xopt(dim+1);
else
  error('Problem of convergence in  inscribe_ball.m');
end


