#include "mex.h"
#include "qhull_a.h"

/* Variable used by io.c */



/*-------------------------------------------------
-internal function prototypes
*/
void print_summary (void);
void qh_printpointid_matlab(FILE *fp, char *string, int dim, pointT *point, int id,int compteurr,double *points_intersect,int size_points_intersect) {
  int k;
  realT r; /*bug fix*/
  
 
  if (!point)
    return;
  if (string) {
   fputs (string, fp);
   if (id != -1)
      fprintf(fp, " p%d: ", id);
  }
  for(k= dim; k--; ) {
    r= *point++;
    if (string){
    fprintf(fp, " %8.4g", r);}
    else{
 	points_intersect[compteurr+((qh hull_dim)-1-k)*size_points_intersect]=r;
    }
  }
} /* printpointid */

/*-------------------------------------------------
-print_summary()
*/
void print_summary (void) {
  facetT *facet;
  vertexT *vertex, **vertexp;
  int k,compt;
  pointT  *point;    

  printf("\n%d vertices and %d facets with normals %d :\n",qh num_vertices,qh num_facets,qh num_points);

  printf("\nNormals \n");
  FORALLfacets {
    for (k=0; k < qh hull_dim; k++) printf ("%5.8f ", facet->normal[k]);
    printf ("\n");
  }
  printf("\nVertices\n");

  printf("\n");
}



/*--------------------------------------------------
-main- derived from call_qhull in user.c

  see program header

  this contains three runs of Qhull for convex hull, Delaunay
  triangulation or Voronoi vertices, and halfspace intersection

*/
void mexFunction(int nOutArray, mxArray *OutArray[], int nInArray,const mxArray *InArray[]){


  int dim;	            /* dimension of points */
  int numpoints;            /* number of points */
 
  boolT ismalloc= False;    /* True if qhull should free points in qh_freeqhull() or reallocation */
  char* flags;          /* option flags for qhull, see qh_opt.htm */
  char flagst[1000];       
  FILE *outfile= stdout;    /* output from qh_produce_output()
			       use NULL to skip qh_produce_output() */
  FILE *errfile= stderr;    /* error messages from qhull code */
  int exitcode;             /* 0 if no error from qhull */
  facetT *facet;	    /* set by FORALLfacets */
  int curlong, totlong;	    /* memory remaining after qh_memfreeshort */
  int i,j,k;
  double *H;
  double *K;
  double *intpoint;
  double*    points_intersect; 

  int numfacets, numsimplicial, numridges, totneighbors, numcoplanars, numtricoplanars;
  int  num;
  facetT  **facetp;
  vertexT *vertex, **vertexp;
  setT *vertices;
  pointT *point, **pointp, *pointtemp;
  coordT *normp, *coordp, *feasiblep;
  facetT *facetlist;
 
  boolT zerodiv;
  int format, string, id;
  realT r; 
  int        size_points_intersect; 
  int        compteurr;
  
  flags=(char *)calloc(10000,sizeof(char) );

  /*################################################################################*/
  /* MATLAB STUFF */

  H           = mxGetPr(InArray[0]);
  K           = mxGetPr(InArray[1]);
  intpoint    = mxGetPr(InArray[2]);
  dim         = mxGetN(InArray[0]);
  numpoints   = mxGetM(InArray[0]);
 
  /*  coordT points[(dim+1)*numpoints]; */
  /* array of coordinates for each point */
  coordT* points;
  coordT** rows;
 


  points=(coordT*)calloc((dim+1)*numpoints,sizeof(coordT));
  
 
  for (k=0; k<dim; k++)  {
    for (j=0; j<numpoints; j++){
      points[k+j*(dim+1)] = H[k*numpoints+j];
    }
  } 
 
  for (j=0; j<numpoints; j++) points[(j+1)*(dim+1)-1]=-K[j]; 
 

  /*################################################################################*/
  /* QHULL STUFF */
  sprintf(flags, "qhull   %s  H0 ", "Fp");
  /*  for (k=0; k<dim; k++){
          sprintf(flagst, " %.4lf",intpoint[k]);
	       strcat(flags,flagst);
	       feasible[k]=intpoint[k];}*/ 
  
  rows=(coordT**)calloc(numpoints,sizeof(coordT*));
 
  for (i=numpoints; i--; ) {
    rows[i]= points+(dim+1)*i;}
  
  exitcode= qh_new_qhull (dim+1, numpoints, points, ismalloc,
                      flags,NULL, errfile); 
   
  qh printoutnum= 0;
  qh_countfacets (qh facet_list,NULL, False, &numfacets, &numsimplicial, 
		  &totneighbors, &numridges, &numcoplanars, &numtricoplanars);
  size_points_intersect =  numfacets;
 
 
  OutArray[0]      = mxCreateDoubleMatrix(numfacets,qh hull_dim, mxREAL);
  points_intersect = mxGetPr(OutArray[0]);

 
  compteurr=0;
  for (i= 0; i < qh_PRINTEND; i++){
    format = qh PRINTout[i];   
    FORALLfacet_(qh facet_list){
      if (facet->visible && qh NEWfacets && format != qh_PRINTfacets)
	break;
      qh printoutnum++;
      switch (format) {
      case qh_PRINTpointintersect:
	if (facet->offset > 0)
	  goto LABELprintinfinite;
	point= coordp= (coordT*)qh_memalloc (qh normal_size);
	normp= facet->normal;
	feasiblep= qh feasible_point;
	if (facet->offset < -qh MINdenom) {
	  for (k= qh hull_dim; k--; )
	    *(coordp++)= (*(normp++) / - facet->offset) + *(feasiblep++);
	}else {
	  for (k= qh hull_dim; k--; ) {
	    *(coordp++)= qh_divzero (*(normp++), facet->offset, qh MINdenom_1,
				     &zerodiv) + *(feasiblep++);
	    if (zerodiv) {
	      qh_memfree (point, qh normal_size);
	      goto LABELprintinfinite;
	    }
	  }
	}
	id= qh_pointid( point);
	
        qh_printpointid_matlab(outfile, False,qh hull_dim , point, id,compteurr, points_intersect,size_points_intersect);
	compteurr++;	
	qh_memfree (point, qh normal_size);
	break;
      LABELprintinfinite:  
	break;
      }
    }  
  }

  /*################################################################################*/
  /*printf("\nInfos %d %d\n",qh hull_dim,size_points_intersect);
   
  for (i=0;i<size_points_intersect; i++){
    for (j=0;j<qh hull_dim; j++){
      printf("%.4f ",  points_intersect[i*(qh hull_dim)+j]);
    }
    printf("\n");
    }*/
  
  if (!exitcode) 
    /*   print_summary();*/
  qh_freeqhull (!qh_ALL);
  qh_memfreeshort (&curlong, &totlong);
  if (curlong || totlong)  /* could also check previous runs */
    fprintf (stderr, "qhull internal warning (user_eg, #3): did not free %d bytes of long memory (%d pieces)\n",
       totlong, curlong);

  free(rows);
  free(points);
  return;

  
} /* main */

