#include "mex.h"
#include "qhull_a.h"

/* Variable used by io.c */



/*-------------------------------------------------
-internal function prototypes
*/
void print_summary (double* hout,double* kout);

/*-------------------------------------------------
-print_summary()
*/
void print_summary (double* hout,double* kout) {
  facetT *facet;
  int k,compt;

  compt = 0;

  FORALLfacets {
    for (k=0; k < qh hull_dim; k++) {
      /* printf ("%6.2g ", facet->normal[k]);
	 printf ("%6.2g ",facet->offset);*/
      hout[compt+k*(qh num_facets)]= facet->normal[k];}
    kout[compt]=-facet->offset;
    compt++;
  }
}



/*--------------------------------------------------
-main- derived from call_qhull in user.c

  see program header

  this contains three runs of Qhull for convex hull, Delaunay
  triangulation or Voronoi vertices, and halfspace intersection

*/
void mexFunction(int nOutArray, mxArray *OutArray[], int nInArray,const mxArray *InArray[]){


  int dim;	            /* dimension of points */
  int numpoints;            /* number of points */
 
  boolT ismalloc= False;    /* True if qhull should free points in qh_freeqhull() or reallocation */
  char flags[250];          /* option flags for qhull, see qh_opt.htm */
  FILE *outfile= stdout;    /* output from qh_produce_output()
			       use NULL to skip qh_produce_output() */
  FILE *errfile= stderr;    /* error messages from qhull code */
  int exitcode;             /* 0 if no error from qhull */
  facetT *facet;	    /* set by FORALLfacets */
  int curlong, totlong;	    /* memory remaining after qh_memfreeshort */
  int i,j,k;
  double *V;
  double*   hout; 
  double*   kout; 

  int numfacets, numsimplicial, numridges, totneighbors, numcoplanars, numtricoplanars;
  int  num;
  facetT  **facetp;
  vertexT *vertex, **vertexp;
  setT *vertices;
  pointT *point, **pointp, *pointtemp;
  coordT *normp, *coordp, *feasiblep;
  facetT *facetlist;
  boolT zerodiv;
  int format, string, id;
  realT r; 
  int        size_points_intersect; 
  int        compteurr;
  

  /*################################################################################*/
  /* MATLAB STUFF */

  V           = mxGetPr(InArray[0]);
  dim         = mxGetN(InArray[0]);
  numpoints   = mxGetM(InArray[0]);
 
  coordT* points; /* array of coordinates for each point */
  /* coordT *rows[numpoints];*/ 
  coordT **rows;

  rows=(coordT**)calloc(numpoints,sizeof(coordT*));
  points=(coordT*)calloc((dim+1)*numpoints,sizeof(coordT));
  
  /*printf("%d %d\n\n",dim,numpoints);*/
  for (k=0; k<dim; k++)  {
    for (j=0; j<numpoints; j++){
      points[k+j*dim] = V[k*numpoints+j];
     }
   } 
 


  /*################################################################################*/
  /* QHULL STUFF */
  /*sprintf (flags, "qhull C-0.0000001 %s", "n");*/
  sprintf (flags, "qhull  %s", "n");

  for (i=numpoints; i--; ) {
    rows[i]= points+dim*i;}
  
  exitcode= qh_new_qhull (dim, numpoints, points, ismalloc,
                      flags,NULL,errfile); 
  numfacets = qh num_facets;


  /*printf("Nb faces %d  \n",qh num_facets);*/
  OutArray[0]      = mxCreateDoubleMatrix(numfacets,qh hull_dim, mxREAL);
  OutArray[1]      = mxCreateDoubleMatrix(numfacets,1, mxREAL);

  hout  = mxGetPr(OutArray[0]);
  kout  = mxGetPr(OutArray[1]);
  print_summary(hout,kout);


  
  if (!exitcode) 
    /*   print_summary();*/
  qh_freeqhull (!qh_ALL);
  qh_memfreeshort (&curlong, &totlong);
  if (curlong || totlong)  /* could also check previous runs */
    fprintf (stderr, "qhull internal warning (user_eg, #3): did not free %d bytes of long memory (%d pieces)\n",
       totlong, curlong);

  free(rows);
  free(points);
  return;

  
} /* main */

