function pm = minkowski_sum(p1,p2)

% Minkowski's sum  of two CONVEX bodies
% (in dimension 2 or 3) described as the convex hull 
% of points 
% 
% Input  : sets of points P1 and P2 (of dimensions n*dim and m*dim)
% Output : points Pm (dimension o*dim)
%
% CALL : PM = MINKOWSKI_SUM(P1,P2)
%
% Example in 2d :  p1  = [-1 -1; 1 -1; 1 1; -1 1];
%                  t   = linspace(0,2*pi,100)';
%                  p2  = [cos(t),sin(t)];
%                  pm  = minkowski_sum(p1,p2);

 


np1  = size(p1,1); 
np2  = size(p2,1); 
dim  = size(p1,2);
nmax = 1e6;
pm   = [];

if (dim<2 | (size(p1,2)~=size(p1,2)) | np1>nmax/10 | np2>nmax/10)
    error('Problem of dimension in minkowski_sum.m');
end

if(np1*np2<=nmax)
   pm      = zeros(np1*np2,dim);
   pt      = repmat(p2,1,np1)';
   pt      = reshape(pt,np1*np2*dim,1);
   pt      = reshape(pt,dim,np1*np2)';    
   pm      = repmat(p1,np2,1) + pt;
else
   pas     = floor(nmax/np2);
   npas    = floor(np1/pas);
   for   i = 1:npas
     p3    = minkowski_sum(p1(((i-1)*pas+1):i*pas,:),p2);
     p4    = [pm;p3];
     if (dim==2)
       K     = convhull(p4(:,1),p4(:,2)); 
       pm    = p4(K,:);
     else  
       K     = convhulln(p4);
       liste = unique([K(:,1);K(:,2);K(:,3)]);
       pm    = p4(liste,:);
     end
   end
   if isempty((npas*pas+1):np1)
   else
      p3    = minkowski_sum(p1((npas*pas+1):np1,:),p2);
      p4    = [pm;p3];
     if (dim==2)
       K     = convhull(p4(:,1),p4(:,2)); 
       pm    = p4(K,:);
     else  
       K     = convhulln(p4);
       liste = unique([K(:,1);K(:,2);K(:,3)]);
       pm    = p4(liste,:);
     end
   end
end



if (dim==2)
   K     = convhull(pm(:,1),pm(:,2)); 
   pm    = pm(K,:);
else
   K     = convhulln(pm);  
   liste = unique([K(:,1);K(:,2);K(:,3)]);
   pm    = pm(liste,:);
end


