function p =  polytope_H2V(H,K,Xstart)

% This function computes the V-representation of a polytop
% (i.e. its extrem points) given by P = {x, Hx<=K} from this H-representation. 
% If only 2 arguments are given, 0 is supposed to be an
% interior point of the polytop. Otherwise, an interior
% point is computed (linear program).
%
% CALL   :  p =  polytope_H2V(H,K,Xinterior)
%

dim  = size(H,2);

if (dim>3  | dim<2)	
    warning('Problem of dimension in polytope_H2V.m ?? : Dimension > 3');
end

Xstart       = zeros(dim,1);

if(nargin>2)
  Xstart     = ones(dim,1);
  options    = optimset('Display','off');
  m          = size(H,1);
  A          = [H,-K,ones(m,1)];
  B          = zeros(m,1);
  infin      = 1e3;
  lb         = -infin*ones(dim+2,1);
  ub         = infin*ones(dim+2,1);
  lb(dim+1)  = 0;
  lb(dim+2)  = 0;
  ub(dim+1)  = 1e3;
  ub(dim+2)  = 1e3;
  f          = zeros(dim+2,1);
  f(dim+2)   = -1;
  Xstart     = linprog(f,A,B,[],[],lb,ub,[],options);
  Xstart     = Xstart(1:dim)/Xstart(dim+1);
  K          = K-H*Xstart;
end


p  =  mex_polytope_H2V(H,K,Xstart);
p  =  p + ones(size(p,1),1)*Xstart';


