function Sps =  steiner_symetrization(A,n,p,H,K)

% This function compute the Steiner symetrization
% of the convex polytop defined by the EXTREMAL 
% points p ( n*3 or n*2) in
% dimension 3 or 2 whith respect to the plan or the line
% containing the point A and normal 
% to the vector n. 
%
% CALL : Sps =  steiner_symetrization(A,n,p,H,K)
%

if(isempty(H) | isempty(K))
 [H,K]    = polytope_V2H(p);
 p        = polytope_H2V(H,K);
end


np        = size(p,1);  
dim       = size(p,2);
n         = n/norm(n);


if (dim<2 | dim>3 | size(A,1)~=1 | size(A,2)~=dim | size(n,1)~=1 | size(n,2)~=dim)
    error('Problem of dimension in steiner_symetrization.m');
end

if(dim==3)
     % Add more extremal points (specific to the dimension 3) 
     newpt = steiner_new_interestc_3d(A,n,p);
     p     = [p;newpt];
     np    = size(p,1);  
     disp(' ');
     disp(['Number of points added        : ',num2str(size(newpt,1))]);
end


Sp        = projection_hyperplan(A,n,p);


param     = zeros(np,1);
smv2      = H*n';
for     k = 1:np 
  smv     = H*(p(k,:)')-K;
 
  indice  = find(abs(smv)<=1e-10);
  vtemp   = smv2(indice);
  if(max(vtemp)>=0)
     if(min(vtemp)<=0)
     else
       indt     = find(smv2<=0);
       param(k) = min(smv(indt)./smv2(indt));
     end
  else
       indt     = find(smv2>=0);
       param(k) = max(smv(indt)./smv2(indt));
  end
end

param     = abs(param)/2;
Sps       = [Sp+(param*ones(1,dim)).*(ones(np,1)*n);Sp-(param*ones(1,dim)).*(ones(np,1)*n)]; 





%*************************************************************
function [newpt,A,B] = steiner_new_interestc_3d(A,n,p)

newpt    = [];

% Compute the edges of the polytop
edge     = edges_matlab(p,[],[]);
ne       = size(edge,1);
np       = size(p,1);


compt        = 1;
newpt        = zeros(2e4,3);
for k = 1:ne
   A                = p(edge(k,1),:);
   B                = p(edge(k,2),:);
   AB               = B-A;
   normk            = [(AB(2)*n(3)-AB(3)*n(2)),(AB(3)*n(1)-AB(1)*n(3)),(AB(1)*n(2)-AB(2)*n(1))];
   scalprod         = (p-(ones(np,1)*A))*normk';
   vv               = scalprod(edge);
   vv(abs(vv)<1e-9) = 0;

   vvv       = vv(:,1).*vv(:,2);
   indices   = find(vvv<0);   
   C         = p(edge(indices,1),:);
   D         = p(edge(indices,2),:); 
   CD        = D-C;
   scalCDn   = CD(:,1)*normk(1)+ CD(:,2)*normk(2)+ CD(:,3)*normk(3);
   scalADn   = (A(1)-D(:,1))*normk(1)+(A(2)-D(:,2))*normk(2)+(A(3)-D(:,3))*normk(3);
   ttt       = -scalADn./scalCDn; 

   normk1    = [(normk(2)*n(3)-normk(3)*n(2)),(normk(3)*n(1)-normk(1)*n(3)),(normk(1)*n(2)-normk(2)*n(1))];
   ptkk      = (ttt*[1 1 1]).*C+((1-ttt)*[1 1 1]).*D;
   scalA     = (ptkk-(ones(size(ptkk,1),1)*A))*normk1';
   scalB     = (ptkk-(ones(size(ptkk,1),1)*B))*normk1';
   vvvk      = scalA.*scalB;
   vvkv(abs(vvvk)<1e-9) = 0;
   indk                 = find(vvvk<0);
   ptkk                 = ptkk(indk,:);
 

 
   sptk                          = size(ptkk,1);
   newpt(compt:(compt+sptk-1),:) = ptkk;  
   compt                         = compt+sptk;

end
newpt        = newpt(1:(compt-1),:);  
