function test_steiner(test,dim)

% Test the steiner_symetrization.m function
%
% CALL : test_steiner(test,dim)
%        test : number of the test case (between 1 and 2)
%        dim  : 2 or 3

n1           = 3e1;
n            = rand(1,dim);
n            = n./(norm(n)*ones(1,dim));
A            = rand(1,dim);

switch(test)
case 1
  p1           = 2*rand(n1,dim)-1;
case 2
  m            = 2e2;
  n1           = m;
  randn('seed',1);
  normaux      = randn(dim,m);
  normaux      = normaux./(ones(dim,1)*(sum(normaux.^2,1).^(0.5)));
  p1           = normaux';
end




[H,K]        = polytope_V2H(p1);
p1           = polytope_H2V(H,K);
tic
p2           = steiner_symetrization(A,n,p1,H,K);
disp(['Time of execution of edge_matlab.m    :  ',num2str(toc),'  seconds'])





if(dim==3)
  disp(' ');
  [volume1,surface] =  volsurf(p1);
  disp(['Volume  of the first polytop      = ',num2str(volume1,16)])
  [volume2,surface] =  volsurf(p2);
  disp(['Volume  of the symetrized polytop = ',num2str(volume2,16)])
  disp(' ');   
  
  % Representation graphique
  close all
  hold on
  borne        = 2;
  t            = [n(2);-n(1)];

  figure(1)
  set(gcf,'Name',['Steiner symetrization','''','s of a polytope'])
  subplot(1,2,1);
  hold on

 
  K1  = convhulln(p1);
  h   = trisurf(K1,p1(:,1),p1(:,2),p1(:,3),ones(size(p1,1),1));
  set(h,'EdgeColor',[0.4 0.8 .9]);
  set(h,'FaceColor',[0.3 0.7 .5]);

  set(h,'FaceAlpha',0.9);
  set(h,'LineStyle','-');

  pc  = 1.5*[1 1 1;-1 1 1;1 -1 1;-1 -1 1;1 1 -1;-1 1 -1;1 -1 -1;-1 -1 -1;];
  pc  = projection_hyperplan(A,n,pc);
  K3  = convhulln(pc);
  h   = trisurf(K3,pc(:,1),pc(:,2),pc(:,3),2*ones(size(pc,1),1));
  set(h,'FaceColor','b');  set(h,'FaceAlpha',0.1);
  set(h,'EdgeColor','w');
  set(h,'LineStyle',':');
  set(h,'LineWidth',0.1);

  axis equal
  axis off
  zoom(1.5)

  subplot(1,2,2);
  hold on

  K2  = convhulln(p2);
  h   = trisurf(K2,p2(:,1),p2(:,2),p2(:,3),ones(size(p1,1),1));
  set(h,'EdgeColor',[0.2 0.8 .2]);
  set(h,'FaceColor',[0.2 0.7 .2]);
  set(h,'FaceAlpha',0.9);
  set(h,'LineStyle','-');

  h   = plot3(p2(:,1),p2(:,2),p2(:,3),'*');
  set(h,'Color',[0.9 0.8 .2]);
  set(h,'MarkerSize',3);
  
  pc  = 1.5*[1 1 1;-1 1 1;1 -1 1;-1 -1 1;1 1 -1;-1 1 -1;1 -1 -1;-1 -1 -1;];
  pc  = projection_hyperplan(A,n,pc);
  K3  = convhulln(pc);
  h   = trisurf(K3,pc(:,1),pc(:,2),pc(:,3),2*ones(size(pc,1),1));set(h,'EdgeColor','none');
  set(h,'FaceColor','b');  set(h,'FaceAlpha',0.1);
  set(h,'EdgeColor','w');set(h,'LineStyle',':');
  set(h,'LineWidth',0.1);

  axis equal
  axis off
    zoom(1.5)
else
  disp(' ');
  [volume1,surface] =  volsurf(p1);
  disp(['Area of the first polytop       = ',num2str(volume1,16)])
  [volume2,surface] =  volsurf(p2);
  disp(['Area of the symetrized polytop  = ',num2str(volume2,16)])
  disp(' ');


  % Representation graphique
  close all
  hold on
  borne        = 2;
  t            = [n(2);-n(1)];

  figure(1)
  set(gcf,'Name',['Steiner symetrization','''','s of a polytope'])
  subplot(1,2,1);
  hold on
  K1         = convhull(p1(:,1),p1(:,2));
  p1         = p1(K1(1:(length(K1)-1)),:);
  h = fill(p1(:,1),p1(:,2),1);
  set(h,'EdgeColor',[0.8 0.3 .8]);
  set(h,'FaceColor',[0.7 0.2 .3]);
  plot(A(1),A(2),'+w');
  plot(A(1),A(2),'+w');
  plot([A(1)-borne*t(1),A(1)+borne*t(1)],[A(2)-borne*t(2),A(2)+borne*t(2)],'r');
  axis equal
  axis off

  subplot(1,2,2);
  hold on
  K2           = convhull(p2(:,1),p2(:,2));
  p2           = p2(K2,:);
  h = fill(p2(:,1),p2(:,2),'b');
  set(h,'EdgeColor',[0.3 0.8 .8]);
  set(h,'FaceColor',[0.2 0.7 .3]);
  plot(A(1),A(2),'+w');
  plot([A(1)-borne*t(1),A(1)+borne*t(1)],[A(2)-borne*t(2),A(2)+borne*t(2)],'r');
  axis equal
  axis off
end

